@@ -1,4 +1,6 @@
-{-# OPTIONS_GHC -XNoImplicitPrelude #-}
+{-# LANGUAGE Unsafe #-}
+{-# LANGUAGE NoImplicitPrelude #-}
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Foreign.Marshal
@@ -14,15 +16,11 @@
 -----------------------------------------------------------------------------
 
 module Foreign.Marshal
-        ( module Foreign.Marshal.Alloc
-        , module Foreign.Marshal.Array
-        , module Foreign.Marshal.Error
-        , module Foreign.Marshal.Pool
-        , module Foreign.Marshal.Utils
+        (
+         -- | The module "Foreign.Marshal" re-exports the safe content in the
+         -- @Foreign.Marshal@ hierarchy:
+          module Foreign.Marshal.Safe
         ) where
 
-import Foreign.Marshal.Alloc
-import Foreign.Marshal.Array
-import Foreign.Marshal.Error
-import Foreign.Marshal.Pool
-import Foreign.Marshal.Utils
+import Foreign.Marshal.Safe
+
