@@ -1,16 +1,18 @@
 {-# LANGUAGE CPP #-}
-{-# OPTIONS_GHC -fno-warn-orphans #-}
-
--- #prune
+{-# OPTIONS_HADDOCK prune #-}
+#if __GLASGOW_HASKELL__ >= 701
+{-# LANGUAGE Trustworthy #-}
+#endif
 
 -- |
 -- Module      : Data.ByteString.Lazy.Char8
--- Copyright   : (c) Don Stewart 2006
+-- Copyright   : (c) Don Stewart 2006-2008
+--               (c) Duncan Coutts 2006-2011
 -- License     : BSD-style
 --
--- Maintainer  : dons@cse.unsw.edu.au
--- Stability   : experimental
--- Portability : non-portable (imports Data.ByteString.Lazy)
+-- Maintainer  : dons00@gmail.com, duncan@community.haskell.org
+-- Stability   : stable
+-- Portability : portable
 --
 -- Manipulate /lazy/ 'ByteString's using 'Char' operations. All Chars will
 -- be truncated to 8 bits. It can be expected that these functions will
@@ -22,6 +24,11 @@
 --
 -- > import qualified Data.ByteString.Lazy.Char8 as C
 --
+-- The Char8 interface to bytestrings provides an instance of IsString
+-- for the ByteString type, enabling you to use string literals, and
+-- have them implicitly packed to ByteStrings.
+-- Use @{-\# LANGUAGE OverloadedStrings \#-}@ to enable this.
+--
 
 module Data.ByteString.Lazy.Char8 (
 
@@ -35,6 +42,8 @@ module Data.ByteString.Lazy.Char8 (
         unpack,                 -- :: ByteString -> String
         fromChunks,             -- :: [Strict.ByteString] -> ByteString
         toChunks,               -- :: ByteString -> [Strict.ByteString]
+        fromStrict,             -- :: Strict.ByteString -> ByteString
+        toStrict,               -- :: ByteString -> Strict.ByteString
 
         -- * Basic interface
         cons,                   -- :: Char -> ByteString -> ByteString
@@ -45,6 +54,7 @@ module Data.ByteString.Lazy.Char8 (
         uncons,                 -- :: ByteString -> Maybe (Char, ByteString)
         last,                   -- :: ByteString -> Char
         tail,                   -- :: ByteString -> ByteString
+        unsnoc,                 -- :: ByteString -> Maybe (ByteString, Char)
         init,                   -- :: ByteString -> ByteString
         null,                   -- :: ByteString -> Bool
         length,                 -- :: ByteString -> Int64
@@ -157,6 +167,9 @@ module Data.ByteString.Lazy.Char8 (
         readInteger,
 
         -- * I\/O with 'ByteString's
+        -- | ByteString I/O uses binary mode, without any character decoding
+        -- or newline conversion. The fact that it does not respect the Handle
+        -- newline mode is considered a flaw and may be changed in a future version.
 
         -- ** Standard input and output
         getContents,            -- :: IO ByteString
@@ -174,22 +187,25 @@ module Data.ByteString.Lazy.Char8 (
         hGet,                   -- :: Handle -> Int64 -> IO ByteString
         hGetNonBlocking,        -- :: Handle -> Int64 -> IO ByteString
         hPut,                   -- :: Handle -> ByteString -> IO ()
+        hPutNonBlocking,        -- :: Handle -> ByteString -> IO ByteString
+        hPutStr,                -- :: Handle -> ByteString -> IO ()
+        hPutStrLn,              -- :: Handle -> ByteString -> IO ()
 
   ) where
 
 -- Functions transparently exported
 import Data.ByteString.Lazy 
-        (ByteString, fromChunks, toChunks
+        (fromChunks, toChunks, fromStrict, toStrict
         ,empty,null,length,tail,init,append,reverse,transpose,cycle
         ,concat,take,drop,splitAt,intercalate,isPrefixOf,group,inits,tails,copy
         ,hGetContents, hGet, hPut, getContents
-        ,hGetNonBlocking
-        ,putStr, putStrLn, interact)
+        ,hGetNonBlocking, hPutNonBlocking
+        ,putStr, hPutStr, interact)
 
 -- Functions we need to wrap.
 import qualified Data.ByteString.Lazy as L
+import qualified Data.ByteString as S (ByteString) -- typename only
 import qualified Data.ByteString as B
-import qualified Data.ByteString.Internal as B
 import qualified Data.ByteString.Unsafe as B
 import Data.ByteString.Lazy.Internal
 
@@ -205,22 +221,19 @@ import Prelude hiding
         ,readFile,writeFile,appendFile,replicate,getContents,getLine,putStr,putStrLn
         ,zip,zipWith,unzip,notElem,repeat,iterate,interact,cycle)
 
-import System.IO            (hClose,openFile,IOMode(..))
+import System.IO            (Handle,stdout,hClose,openBinaryFile,IOMode(..))
 #ifndef __NHC__
 import Control.Exception    (bracket)
 #else
 import IO                   (bracket)
 #endif
 
-#if __GLASGOW_HASKELL__ >= 608
-import Data.String
-#endif
-
 #define STRICT1(f) f a | a `seq` False = undefined
 #define STRICT2(f) f a b | a `seq` b `seq` False = undefined
 #define STRICT3(f) f a b c | a `seq` b `seq` c `seq` False = undefined
 #define STRICT4(f) f a b c d | a `seq` b `seq` c `seq` d `seq` False = undefined
 #define STRICT5(f) f a b c d e | a `seq` b `seq` c `seq` d `seq` e `seq` False = undefined
+#define STRICT5_(f) f a b c d _ | a `seq` b `seq` c `seq` d `seq` False = undefined
 
 ------------------------------------------------------------------------
 
@@ -229,20 +242,16 @@ singleton :: Char -> ByteString
 singleton = L.singleton . c2w
 {-# INLINE singleton #-}
 
-#if __GLASGOW_HASKELL__ >= 608
-instance IsString ByteString where
-    fromString = pack
-    {-# INLINE fromString #-}
-#endif
-
 -- | /O(n)/ Convert a 'String' into a 'ByteString'. 
 pack :: [Char] -> ByteString
-pack = L.pack. List.map c2w
+pack = packChars
 
 -- | /O(n)/ Converts a 'ByteString' to a 'String'.
 unpack :: ByteString -> [Char]
-unpack = List.map w2c . L.unpack
-{-# INLINE unpack #-}
+unpack = unpackChars
+
+infixr 5 `cons`, `cons'` --same as list (:)
+infixl 5 `snoc`
 
 -- | /O(1)/ 'cons' is analogous to '(:)' for lists.
 cons :: Char -> ByteString -> ByteString
@@ -285,6 +294,14 @@ uncons bs = case L.uncons bs of
                   Just (w, bs') -> Just (w2c w, bs')
 {-# INLINE uncons #-}
 
+-- | /O(n\/c)/ Extract the 'init' and 'last' of a ByteString, returning Nothing
+-- if it is empty.
+unsnoc :: ByteString -> Maybe (ByteString, Char)
+unsnoc bs = case L.unsnoc bs of
+                  Nothing -> Nothing
+                  Just (bs', w) -> Just (bs', w2c w)
+{-# INLINE unsnoc #-}
+
 -- | /O(1)/ Extract the last element of a packed string, which must be non-empty.
 last :: ByteString -> Char
 last = w2c . L.last
@@ -666,7 +683,7 @@ lines (Chunk c0 cs0) = loop0 c0 cs0
 
     -- the common special case where we have no existing chunks of
     -- the current line
-    loop0 :: B.ByteString -> ByteString -> [ByteString]
+    loop0 :: S.ByteString -> ByteString -> [ByteString]
     loop0 c cs =
         case B.elemIndex (c2w '\n') c of
             Nothing -> case cs of
@@ -683,7 +700,7 @@ lines (Chunk c0 cs0) = loop0 c0 cs0
 
     -- the general case when we are building a list of chunks that are
     -- part of the same line
-    loop :: B.ByteString -> [B.ByteString] -> ByteString -> [ByteString]
+    loop :: S.ByteString -> [S.ByteString] -> ByteString -> [ByteString]
     loop c line cs =
         case B.elemIndex (c2w '\n') c of
             Nothing ->
@@ -763,9 +780,8 @@ readInt (Chunk x xs) = case w2c (B.unsafeHead x) of
     _   -> loop False 0 0 x xs
 
     where loop :: Bool -> Int -> Int
-                -> B.ByteString -> ByteString -> Maybe (Int, ByteString)
-          {-# INLINE loop #-}
-          STRICT5(loop)
+                -> S.ByteString -> ByteString -> Maybe (Int, ByteString)
+          STRICT5_(loop)
           loop neg i n c cs
               | B.null c = case cs of
                              Empty          -> end  neg i n c  cs
@@ -810,8 +826,8 @@ readInteger (Chunk c0 cs0) =
                 | otherwise              -> Nothing
 
           loop :: Int -> Int -> [Integer]
-               -> B.ByteString -> ByteString -> (Integer, ByteString)
-          STRICT5(loop)
+               -> S.ByteString -> ByteString -> (Integer, ByteString)
+          STRICT5_(loop)
           loop d acc ns c cs
               | B.null c = case cs of
                              Empty          -> combine d acc ns c cs
@@ -840,25 +856,34 @@ readInteger (Chunk c0 cs0) =
           end n c cs = let c' = chunk c cs
                         in c' `seq` (n, c')
 
--- | Read an entire file /lazily/ into a 'ByteString'. Use 'text mode'
--- on Windows to interpret newlines
+-- | Read an entire file /lazily/ into a 'ByteString'.
 readFile :: FilePath -> IO ByteString
-readFile f = openFile f ReadMode >>= hGetContents
+readFile f = openBinaryFile f ReadMode >>= hGetContents
 
 -- | Write a 'ByteString' to a file.
 writeFile :: FilePath -> ByteString -> IO ()
-writeFile f txt = bracket (openFile f WriteMode) hClose
+writeFile f txt = bracket (openBinaryFile f WriteMode) hClose
     (\hdl -> hPut hdl txt)
 
 -- | Append a 'ByteString' to a file.
 appendFile :: FilePath -> ByteString -> IO ()
-appendFile f txt = bracket (openFile f AppendMode) hClose
+appendFile f txt = bracket (openBinaryFile f AppendMode) hClose
     (\hdl -> hPut hdl txt)
 
 
+-- | Write a ByteString to a handle, appending a newline byte
+--
+hPutStrLn :: Handle -> ByteString -> IO ()
+hPutStrLn h ps = hPut h ps >> hPut h (L.singleton 0x0a)
+
+-- | Write a ByteString to stdout, appending a newline byte
+putStrLn :: ByteString -> IO ()
+putStrLn = hPutStrLn stdout
+
+
 -- ---------------------------------------------------------------------
 -- Internal utilities
 
 -- reverse a list of possibly-empty chunks into a lazy ByteString
-revChunks :: [B.ByteString] -> ByteString
+revChunks :: [S.ByteString] -> ByteString
 revChunks cs = List.foldl' (flip chunk) Empty cs
